@@ -15,9 +15,9 @@ sub UNITCHECK {				## restore %SIG after compilation
 package Net::DNS::RR::SIG;
 
 #
-# $Id: SIG.pm 1726 2018-12-15 12:59:56Z willem $
+# $Id: SIG.pm 1747 2019-06-27 15:02:31Z willem $
 #
-our $VERSION = (qw$LastChangedRevision: 1726 $)[1];
+our $VERSION = (qw$LastChangedRevision: 1747 $)[1];
 
 
 use strict;
@@ -44,13 +44,23 @@ use constant DEBUG => 0;
 
 use constant UTIL => defined eval 'use Scalar::Util 1.25; 1;';
 
-# IMPORTANT: Distros MUST NOT create dependencies on Net::DNS::SEC	(Prohibited in many territories)
-use constant EXISTS => join '', qw(r e q u i r e);		# Defeat static analysers and grep
-use constant DNSSEC => defined( eval join ' ', EXISTS, 'Net::DNS::SEC::Private' );
-use constant ACTIVE => DNSSEC && $INC{'Net/DNS/SEC.pm'};	# Discover how we got here
+# IMPORTANT: Downstream distros MUST NOT create dependencies on Net::DNS::SEC	(strong crypto prohibited in many territories)
+use constant ACTIVE => defined $INC{'Net/DNS/SEC.pm'};		# Discover how we got here, without exposing any crypto
+								# Discourage static code analysers and casual greppers
+use constant DNSSEC => ACTIVE && defined eval join '', qw(r e q u i r e), ' Net::DNS', qw(:: SEC :: Private);
 
-my ($RSA) = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::RSA';
-my ($DSA) = grep ACTIVE && defined( eval join ' ', EXISTS, $_ ), 'Net::DNS::SEC::DSA';
+my @index;
+if (DNSSEC) {
+	my $key = new Net::DNS::RR( type => 'DNSKEY' );
+	my @arg = ( '', $key, '' );				# Grotesquely inefficient; but API not changing anytime soon
+	foreach my $class ( map "Net::DNS::SEC::$_", qw(RSA DSA ECCGOST ECDSA EdDSA) ) {
+		push @index, map eval { $key->algorithm($_); $class->verify(@arg); ( $_ => $class ) }, ( 1 .. 25 )
+				if eval join '', qw(r e q u i r e), " $class";
+	}
+}
+
+my %DNSSEC_verify = @index;
+my %DNSSEC_siggen = @index;
 
 my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 
@@ -91,9 +101,9 @@ sub _encode_rdata {			## encode rdata as wire-format octet string
 sub _format_rdata {			## format rdata portion of RR string.
 	my $self = shift;
 
-	my $signame = $self->{signame} || return '';
+	my $sname = $self->{signame} || return '';
 	my @sig64 = split /\s+/, MIME::Base64::encode( $self->sigbin );
-	my @rdata = ( map( $self->$_, @field ), $signame->string, @sig64 );
+	my @rdata = ( map( $self->$_, @field ), $sname->string, @sig64 );
 }
 
 
@@ -149,7 +159,7 @@ sub _defaults {				## specify RR attribute default values
 	my %algbyval = reverse @algbyname;
 
 	my @algrehash = map /^\d/ ? ($_) x 3 : do { s/[\W_]//g; uc($_) }, @algbyname;
-	my %algbyname = @algrehash;    # work around broken cperl
+	my %algbyname = @algrehash;				# work around broken cperl
 
 	sub _algbyname {
 		my $arg = shift;
@@ -167,18 +177,6 @@ sub _defaults {				## specify RR attribute default values
 }
 
 
-my %DNSSEC_sign = (
-	1  => $RSA,
-	3  => $DSA,
-	5  => $RSA,
-	6  => $DSA,
-	7  => $RSA,
-	8  => $RSA,
-	10 => $RSA,
-	);
-
-my %DNSSEC_verify = %DNSSEC_sign;
-
 my %siglen = (
 	1  => 128,
 	3  => 41,
@@ -187,12 +185,17 @@ my %siglen = (
 	7  => 256,
 	8  => 256,
 	10 => 256,
+	12 => 64,
+	13 => 64,
+	14 => 96,
+	15 => 64,
+	16 => 114,
 	);
 
 
 sub _size {				## estimate encoded size
-	my $self = shift;
-	my $clone = bless {%$self}, ref($self);			   # shallow clone
+	my $self  = shift;
+	my $clone = bless {%$self}, ref($self);			# shallow clone
 	$clone->sigbin( 'x' x $siglen{$self->algorithm} );
 	length $clone->encode();
 }
@@ -293,7 +296,7 @@ sub signature { &sig; }
 
 sub create {
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $class, $data, $priv_key, %etc ) = @_;
 
@@ -316,8 +319,6 @@ sub create {
 		$self->{sigexpiration} = $self->{siginception} + $self->{sigval}
 				unless $self->{sigexpiration};
 
-		croak 'No "use Net::DNS::SEC" declaration in application code' unless ACTIVE;
-
 		$self->_CreateSig( $self->_CreateSigData($data), $private ) if $data;
 
 		$self->{private} = $private unless $data;	# mark packet for SIG0 generation
@@ -337,14 +338,14 @@ sub verify {
 	# of keys.
 
 	unless (DNSSEC) {
-		croak 'Net::DNS::SEC support not available';
+		croak qq[No "use Net::DNS::SEC" declaration in application code];
 	} else {
 		my ( $self, $dataref, $keyref ) = @_;
 
 		if ( my $isa = ref($dataref) ) {
 			print '$dataref argument is ', $isa, "\n" if DEBUG;
-			croak '$dataref can not be ', $isa unless $isa =~ /^Net::DNS::/;
-			croak '$dataref can not be ', $isa unless $dataref->isa('Net::DNS::Packet');
+			croak '$dataref must be scalar or a Net::DNS::Packet'
+					unless $isa =~ /Net::DNS/ && $dataref->isa('Net::DNS::Packet');
 		}
 
 		print '$keyref argument is of class ', ref($keyref), "\n" if DEBUG;
@@ -400,8 +401,6 @@ sub verify {
 		# The data that is to be verified
 		my $sigdata = $self->_CreateSigData($dataref);
 
-		croak 'No "use Net::DNS::SEC" declaration in application code' unless ACTIVE;
-
 		my $verified = $self->_VerifySig( $sigdata, $keyref ) || return 0;
 
 		# time to do some time checking.
@@ -513,7 +512,6 @@ sub _CreateSigData {
 			$message = $hbin . substr $data, length $hbin;
 		}
 
-		my @field = qw(typecovered algorithm labels orgttl sigexpiration siginception keytag);
 		my $sigdata = pack 'n C2 N3 n a*', @{$self}{@field}, $self->{signame}->encode;
 		print "\npreamble\t", unpack( 'H*', $sigdata ), "\nrawdata\t", unpack( 'H100', $message ), " ...\n"
 				if DEBUG;
@@ -529,11 +527,11 @@ sub _CreateSig {
 		my $self = shift;
 
 		my $algorithm = $self->algorithm;
-		my $class     = $DNSSEC_sign{$algorithm};
+		my $class     = $DNSSEC_siggen{$algorithm};
 
 		eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$self->sigbin( $class->sign(@_) );
+			return $self->sigbin( $class->sign(@_) ) if $class;
+			die "algorithm $algorithm not supported\n";
 		} || croak "${@}signature generation failed";
 	}
 }
@@ -547,8 +545,8 @@ sub _VerifySig {
 		my $class     = $DNSSEC_verify{$algorithm};
 
 		my $retval = eval {
-			die "algorithm $algorithm not supported" unless $class;
-			$class->verify( @_, $self->sigbin );
+			return $class->verify( @_, $self->sigbin ) if $class;
+			die "algorithm $algorithm not supported\n";
 		};
 
 		unless ($retval) {
@@ -558,7 +556,7 @@ sub _VerifySig {
 		}
 
 		# uncoverable branch true	# bug in Net::DNS::SEC or dependencies
-		croak 'unknown error in $class->verify' unless $retval == 1;
+		croak "unknown error in $class->verify" unless $retval == 1;
 		print "\nalgorithm $algorithm verification successful\n" if DEBUG;
 		return 1;
 	}
@@ -704,8 +702,8 @@ that comes with the ISC BIND distribution.
 The optional remaining arguments consist of ( name => value ) pairs
 as follows:
 
-	sigin  => 20181201010101,	# signature inception
-	sigex  => 20181201011101,	# signature expiration
+	sigin  => 20191201010101,	# signature inception
+	sigex  => 20191201011101,	# signature expiration
 	sigval => 10,			# validity window (minutes)
 
 The sigin and sigex values may be specified as Perl time values or as
